"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processInBatches = processInBatches;
exports.retryWithBackoff = retryWithBackoff;
const Logger_1 = __importDefault(require("../utils/Logger"));
async function processInBatches(processBatch, handler, options) {
    const batchSize = options?.batchSize ?? 1000;
    const maxConcurrentBatches = 3;
    let offset = 0;
    let activeBatches = [];
    while (true) {
        const items = await processBatch(offset, batchSize);
        if (!items || items.length === 0) {
            break;
        }
        const batchTask = handler(items);
        activeBatches.push(batchTask);
        if (activeBatches.length >= maxConcurrentBatches) {
            await Promise.race(activeBatches);
            activeBatches = activeBatches.filter((b) => !b.finally);
        }
        offset += items.length;
    }
    await Promise.all(activeBatches);
}
async function retryWithBackoff(operation, maxRetries = 5, baseDelay = 500) {
    let attempt = 0;
    while (attempt <= maxRetries) {
        try {
            return await operation();
        }
        catch (error) {
            if (attempt === maxRetries) {
                Logger_1.default.error(`Max retries reached. Operation failed: ${error.message}`);
                throw error;
            }
            const delay = baseDelay * Math.pow(2, attempt);
            Logger_1.default.log(`Retrying after ${delay}ms... Attempt ${attempt + 1}`);
            await new Promise((resolve) => setTimeout(resolve, delay));
            attempt++;
        }
    }
    throw new Error("This should never happen, as we throw after max retries.");
}
//# sourceMappingURL=BatchProcessor.js.map