"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EmailNotifier = void 0;
const nodemailer_1 = __importDefault(require("nodemailer"));
const RateLimiter_1 = require("../../core/RateLimiter");
const Logger_1 = __importDefault(require("../../utils/Logger"));
class EmailNotifier {
    config;
    transporter;
    rateLimiter;
    constructor(config) {
        this.config = config;
        this.transporter = nodemailer_1.default.createTransport({
            host: config.host,
            port: config.port,
            secure: config.secure,
            auth: config.auth,
        });
        this.rateLimiter = new RateLimiter_1.RateLimiter(config.maxEmailsPerSecond || 10, 1000);
    }
    async send(users, meta) {
        const results = [];
        const maxConcurrentEmails = 3;
        let activeSends = [];
        for (let i = 0; i < users.length; i++) {
            const email = users[i];
            const emailMeta = meta[i] || {};
            const sendTask = this.rateLimiter.schedule(async () => {
                try {
                    const emailOptions = {
                        from: this.config.from,
                        to: email,
                        subject: emailMeta.subject || "No Subject",
                        text: emailMeta.text || "",
                        html: emailMeta.html || "",
                        ...(emailMeta.attachments
                            ? { attachments: emailMeta.attachments }
                            : {}),
                    };
                    const info = await this.transporter.sendMail(emailOptions);
                    Logger_1.default.log(`📨 Email sent to ${email}: ${info.messageId}`);
                    results.push({ status: "success", recipient: email, response: info });
                }
                catch (err) {
                    Logger_1.default.error(`❌ Email Error (Recipient ${email}):`, err.message);
                    results.push({
                        status: "failed",
                        recipient: email,
                        error: err.message,
                    });
                }
            });
            activeSends.push(sendTask);
            if (activeSends.length >= maxConcurrentEmails) {
                await Promise.race(activeSends);
                activeSends = activeSends.filter((task) => !task.finally);
            }
        }
        await Promise.all(activeSends);
        return results;
    }
}
exports.EmailNotifier = EmailNotifier;
//# sourceMappingURL=EmailNotifier.js.map