"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FirebaseNotifier = void 0;
const firebase_admin_1 = __importDefault(require("firebase-admin"));
const RateLimiter_1 = require("../../core/RateLimiter");
const Logger_1 = __importDefault(require("../../utils/Logger"));
class FirebaseNotifier {
    config;
    static initialized = false;
    rateLimiter;
    constructor(config) {
        this.config = config;
        this.initFirebase();
        this.rateLimiter = new RateLimiter_1.RateLimiter(config.maxMessagesPerSecond || 500, 1000);
    }
    initFirebase() {
        if (!FirebaseNotifier.initialized) {
            if (firebase_admin_1.default.apps.length === 0) {
                firebase_admin_1.default.initializeApp({
                    credential: firebase_admin_1.default.credential.cert(this.config.serviceAccount),
                });
                Logger_1.default.log("✅ Firebase initialized successfully.");
            }
            FirebaseNotifier.initialized = true;
        }
    }
    async send(userIds, meta) {
        const messaging = firebase_admin_1.default.messaging();
        const results = [];
        for (let i = 0; i < userIds.length; i++) {
            const userId = userIds[i];
            const userMeta = meta?.[i] || {};
            await this.rateLimiter.schedule(async () => {
                try {
                    const response = await messaging.send({
                        token: userId,
                        notification: {
                            title: userMeta.title || "Default Notification",
                            body: userMeta.body || "",
                        },
                        ...(userMeta.data ? { data: userMeta.data } : {}),
                    });
                    Logger_1.default.log(`📨 Firebase notification sent to ${userId}`);
                    results.push({
                        status: "success",
                        recipient: userId,
                        response,
                    });
                }
                catch (error) {
                    Logger_1.default.error(`❌ Firebase Error (Token: ${userId}): ${error.message}`);
                    results.push({
                        status: "failed",
                        recipient: userId,
                        error: error.message,
                    });
                }
            });
        }
        return results;
    }
    chunkArray(array, chunkSize) {
        return Array.from({ length: Math.ceil(array.length / chunkSize) }, (_, i) => array.slice(i * chunkSize, i * chunkSize + chunkSize));
    }
}
exports.FirebaseNotifier = FirebaseNotifier;
//# sourceMappingURL=FirebaseNotifier.js.map