"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebPushNotifier = void 0;
const web_push_1 = __importDefault(require("web-push"));
const RateLimiter_1 = require("../../core/RateLimiter");
const Logger_1 = __importDefault(require("../../utils/Logger"));
class WebPushNotifier {
    config;
    rateLimiter;
    constructor(config) {
        this.config = config;
        web_push_1.default.setVapidDetails("mailto:" + this.config.contactEmail, this.config.publicKey, this.config.privateKey);
        this.rateLimiter = new RateLimiter_1.RateLimiter(config.maxMessagesPerSecond || 50, 1000);
    }
    async send(userIds, meta) {
        const subscriptions = userIds.map((id) => JSON.parse(id));
        const results = [];
        const maxConcurrentSends = 5;
        let activeSends = [];
        for (let i = 0; i < subscriptions.length; i++) {
            const subscription = subscriptions[i];
            const pushMeta = meta[i] ?? { title: "Notification", body: "", data: {} };
            const sendTask = this.rateLimiter.schedule(async () => {
                try {
                    const pushPayload = JSON.stringify({
                        title: pushMeta.title || "Notification",
                        body: pushMeta.body || "",
                        data: pushMeta.data || {},
                    });
                    const requestOptions = JSON.parse(JSON.stringify({
                        TTL: pushMeta.TTL,
                        vapidDetails: pushMeta.vapidDetails,
                        headers: pushMeta.headers,
                    }));
                    const response = await web_push_1.default.sendNotification(subscription, pushPayload, requestOptions);
                    Logger_1.default.log(`📨 Web Push sent successfully to ${subscription.endpoint}`);
                    results.push({
                        status: "success",
                        recipient: subscription.endpoint,
                        response,
                    });
                }
                catch (error) {
                    Logger_1.default.error(`❌ Web Push Error (Recipient ${subscription.endpoint}):`, error.message);
                    results.push({
                        status: "failed",
                        recipient: subscription.endpoint,
                        error: error.message,
                    });
                }
            });
            activeSends.push(sendTask);
            if (activeSends.length >= maxConcurrentSends) {
                const completedTask = await Promise.race(activeSends);
                activeSends = activeSends.filter((p) => p !== completedTask);
            }
        }
        await Promise.all(activeSends);
        return results;
    }
}
exports.WebPushNotifier = WebPushNotifier;
//# sourceMappingURL=WebPushNotifier.js.map