"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dispatchNotifications = dispatchNotifications;
const RedisClient_1 = require("./utils/RedisClient");
const BatchProcessor_1 = require("./core/BatchProcessor");
const QueueManager_1 = require("./core/QueueManager");
const WorkerManager_1 = require("./core/WorkerManager");
const FirebaseNotifier_1 = require("./jobs/channels/FirebaseNotifier");
const TelegramNotifier_1 = require("./jobs/channels/TelegramNotifier");
const EmailNotifier_1 = require("./jobs/channels/EmailNotifier");
const WebPushNotifier_1 = require("./jobs/channels/WebPushNotifier");
const NotifierRegistry_1 = require("./core/NotifierRegistry");
const Logger_1 = __importDefault(require("./utils/Logger"));
const RateLimiter_1 = require("./core/RateLimiter");
async function dispatchNotifications(options) {
    RedisClient_1.RedisClient.setInstance(options.redisInstance);
    Logger_1.default.enableLogging(options.loggingEnabled ?? true);
    let notifier;
    if (options.customNotifier) {
        notifier = options.customNotifier;
    }
    else {
        switch (options.notifierType) {
            case "firebase":
                notifier = new FirebaseNotifier_1.FirebaseNotifier(options.notifierOptions);
                break;
            case "telegram":
                notifier = new TelegramNotifier_1.TelegramNotifier(options.notifierOptions);
                break;
            case "email":
                notifier = new EmailNotifier_1.EmailNotifier(options.notifierOptions);
                break;
            case "web":
                notifier = new WebPushNotifier_1.WebPushNotifier(options.notifierOptions);
                break;
            default:
                throw new Error("Unsupported notifier type");
        }
    }
    NotifierRegistry_1.NotifierRegistry.register(options.notifierType, notifier);
    const rateLimiter = options.maxQueriesPerSecond
        ? new RateLimiter_1.TokenBucketRateLimiter(options.maxQueriesPerSecond)
        : null;
    await (0, BatchProcessor_1.processInBatches)(async (offset, limit) => {
        if (rateLimiter) {
            await rateLimiter.acquire();
        }
        return await (0, BatchProcessor_1.retryWithBackoff)(() => options.dbQuery(offset, limit));
    }, async (records) => {
        const userIds = records.map(options.mapRecordToUserId);
        await QueueManager_1.QueueManager.enqueueJob(options.queueName, options.jobName, {
            userIds,
            channel: options.notifierType,
            meta: records.map((record) => options.meta(record)),
            trackResponses: options.trackResponses,
            trackingKey: options.trackingKey || "notifications:stats",
        });
    }, {
        batchSize: options.batchSize,
        maxQueriesPerSecond: options.maxQueriesPerSecond,
    });
    if (options.startWorker) {
        new WorkerManager_1.WorkerManager({ queueName: options.queueName });
    }
}
//# sourceMappingURL=runBatchNotificationProcessor.js.map